exception Error of Lexing.position * Lexing.position * string

let print_positions pos1 pos2 =
    if pos1.Lexing.pos_fname <> "" then
      begin
        assert (pos1.Lexing.pos_fname = pos2.Lexing.pos_fname);
        print_string ("file '" ^ pos1.Lexing.pos_fname ^ "', ");
      end;
    let lnum1 = pos1.Lexing.pos_lnum in
    let lnum2 = pos2.Lexing.pos_lnum in
    if lnum1 = lnum2 then assert (pos1.Lexing.pos_bol = pos2.Lexing.pos_bol);
    let column1 = pos1.Lexing.pos_cnum - pos1.Lexing.pos_bol in
    let column2 = pos2.Lexing.pos_cnum - pos2.Lexing.pos_bol in
    if lnum1 = lnum2 then
      begin
        print_string ("line " ^ string_of_int lnum1 ^ ", ");
        if column1 = column2 then
            print_string ("character " ^ string_of_int column1)
        else
            (print_string "characters ";
             print_int column1;
             print_string "-";
             print_int column2)
      end
    else
      begin
        print_string ("line " ^ string_of_int lnum1);
        print_string " character ";
        print_int column1;
        print_string " - ";
        print_string ("line " ^ string_of_int lnum2);
        print_string " character ";
        print_int column2
      end

let handle f =
    try
        f ()
    with
        Error (pos1, pos2, message) ->
            print_string "Syntax error at ";
            print_positions pos1 pos2;
            print_string ":";
            print_newline ();
            print_endline message;
            exit 1
